
subroutine CalcEnergy(Pos, L, bondscreen, lambda0, PEnergy, Dim, NAtom, r00, charge0)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:28) :: r00, lambda0, charge0
    real(8), intent(in) :: L, bondscreen
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 5000., r0 = 1.
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d, d2, id2, id6, id12, charge, lambda, o, rc2
    integer :: i, j
    rc2 = 2.0 ** (1.0/3.0)
    PEnergy = 0.
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            lambda = 0
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            charge = charge0(i) * charge0(j)
            if (charge /= 0) then
                lambda = (abs(lambda0(i))*abs(lambda0(j)))**(1.0/2.0)
            end if
            d = sqrt(sum(rij * rij))
            PEnergy = PEnergy + charge*lambda/d*exp(-1.0*d*bondscreen)
            !bonded pair potential
            if ( j == i + 1 ) then 
                PEnergy = PEnergy + k/2. * (d - r0) * (d - r0)
            !non-bonded pair potential
            else
                d2 = sum(rij * rij)
                if (d2 > rc2) then
                    cycle
                end if
                o = r00(i)+r00(j)
                id2 = o / d2             !inverse squared distance
                id6 = id2 * id2 * id2     !inverse sixth distance
                id12 = id6 * id6          !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + 1.
            endif
        enddo
    enddo
end subroutine


subroutine CalcEnergyForces(Pos, L, bondscreen, lambda0, PEnergy, Forces, Dim, NAtom, r00, charge0)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:28) :: r00, lambda0, charge0
    real(8), intent(in) :: L, bondscreen
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: k = 5000., r0 = 1.
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d, d2, id2, id6, id12, charge, lambda, o, rc2
    integer :: i, j
    rc2 = 2.0 ** (1.0/3.0)
    PEnergy = 0.
    Forces = 0.
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            lambda = 0
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            charge = charge0(i) * charge0(j)
            if (charge /= 0) then
                lambda = (abs(lambda0(i))*abs(lambda0(j)))**(1.0/2.0)
            end if
            d = sqrt(sum(rij * rij))
            PEnergy = PEnergy + charge*lambda/d*exp(-1.0*d*bondscreen)
            Fij = rij * (-1.0*charge*lambda*(1.0/(d*d*d)+bondscreen/(d*d))*exp(-1.0*d*bondscreen))
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
            if (j == i + 1) then 
                PEnergy = PEnergy + k/2.0 * (d - r0) * (d - r0)
                Fij = rij * k * (1-r0/d)
            else
                d2 = sum(rij * rij)                
                if (d2 > rc2) then
                    cycle
                end if
                o = r00(i)+r00(j)
                id2 = o / d2             !inverse squared distance
                id6 = id2 * id2 * id2     !inverse sixth distance
                id12 = id6 * id6          !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + 1.
                Fij = rij * ((-48. * id12 + 24. * id6) * id2)
            endif
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij
        enddo
    enddo
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, L, bondscreen, lambda0, dt, KEnergy, PEnergy, Dim, NAtom, r00, charge0)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in) :: L, bondscreen, dt
    real(8), intent(in), dimension(0:28) :: lambda0, r00, charge0
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, L, bondscreen, lambda0, PEnergy, Accel, Dim, NAtom, r00, charge0)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine
